function [A_aux,B_aux,C_aux,als_vec] = complex_als(Y,iter,A_0,B_0,C_0)
%-------------------------------------------------------------------------%
% [A_aux,B_aux,C_aux,als_vec] = complex(Y_1,iter,A_0,B_0,C_0)
%
% ALS algorithm for CP decomposition with complex factors. 
%
% The inputs are:
% 
% - Y_1         : first data matrix;
% - iter        : maximum number of iterates;
% - sigma_n     : vector with noise standard deviations;
% - A_0,...     : initial factors for the first model.
%
% The outputs are:
% 
% - A,...       : factors for the first model.
% - als_vec : vector with the evolution of the MAP objective function.
%
% List of updates                 -     16/02/2016  -     R. C. Farias
%                                       Creation of the file 
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Number of iterations for each evaluation of the objective
iter_als  =     1;
% Size of the filter for the relative error decrease
n_diff_als=     3;
% Stopping criteria parameters
stop_diff =     eps;
stop_als  =     eps;
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Maximum number of iterations (/iter_als)
iter      =     ceil(iter/iter_als);
% Size of the tensor
dim       =     size(Y);
% Index for n-mode matricization
J         =     prod(dim)./dim;

% Data normalization
norm_Y    =     sum(sum(sum(conj(Y).*Y)));
Y         =     Y/sqrt(norm_Y);
% Three different unfoldings of the tensor
Y_mode    =     cell(3);
% Data tensors unfolding
% First tensor
Y_mode{1}       =     reshape(permute(Y,[1,2,3]),dim(1),J(1));  
Y_mode{2}       =     reshape(permute(Y,[2,1,3]),dim(2),J(2));
Y_mode{3}       =     reshape(permute(Y,[3,1,2]),dim(3),J(3));
%-------------------------------------------------------------------------%

%-----------------------Algorithm initialization--------------------------%
% Filter for the relative error decrease
diff_als_filt   =     1000*ones(1,n_diff_als);
% Storing initial condition on the compressed space
A_aux     =     A_0;
B_aux     =     B_0;
C_aux     =     C_0/sqrt(norm_Y);
X_aux     =     A_aux*transpose(kr(C_aux,B_aux));

% Vector of squared norms of the error
als_vec   =           zeros(1,iter);
% Initial MAP objective value
als_old   =     sum(conj(Y(:)-X_aux(:)).*(Y(:)-X_aux(:)));
% Vector of squared norms of the error
als_vec(1)=           als_old;
%-------------------------------------------------------------------------%

fprintf('\n Complex ALS \n\n')

%--------------------------Algorithm iterate------------------------------%
t         =     0;

while     (mean(diff_als_filt)>stop_diff &&  t<iter ...
          &&  als_old>stop_als   )

    % Iteration counter
    t=t+1;
    
    % Complex ALS (10 iterates)
    for   i           =     1:iter_als
        
    % A factor update
    temp_CB     =     (transpose(C_aux)*conj(C_aux)).*...
                      (transpose(B_aux)*conj(B_aux));
    A_aux       =     (Y_mode{1}*conj(kr(C_aux,B_aux)))/temp_CB;
    norm_A            =     sqrt(sum(abs(A_aux).^2));
    C_aux             =     C_aux.*repmat(norm_A,dim(3),1);
    A_aux             =     A_aux.*repmat(1./norm_A,dim(1),1);
    
    % B factor update
    temp_CA     =     (transpose(C_aux)*conj(C_aux)).*...
                      (transpose(A_aux)*conj(A_aux));
    B_aux       =     (Y_mode{2}*conj(kr(C_aux,A_aux)))/temp_CA;
    norm_B            =     sqrt(sum(abs(B_aux).^2));
    B_aux             =     B_aux.*repmat(1./norm_B,dim(2),1);
    
    % C factor update
    temp_BA     =     (transpose(B_aux)*conj(B_aux)).*...
                      (transpose(A_aux)*conj(A_aux));
    Y_BA        =     Y_mode{3}*conj(kr(B_aux,A_aux));
    C_aux       =     Y_BA/temp_BA;
    end
    
    % Objective value
    %res         =     Y_mode{1}-A_aux*transpose(kr(C_aux,B_aux));
    als_obj     =     sum(sum(conj(res).*res));
    als_obj     =     real(1-2*real(sum(sum(Y_BA.*conj(C_aux))))...
                      +sum(sum((transpose(C_aux)*conj(C_aux)).*temp_BA)));
    
    % Evaluation of the relative error decrease
    diff_als_filt(1:end-1)  =     diff_als_filt(2:end);
    diff_als_filt(end)      =     abs((als_obj-als_old)/als_vec(1));
    
    % Storing the last error
    als_old     =     als_obj;
    als_vec(t+1)=     als_obj;
    if mod(t*iter_als,1)  ==    0
    % Printing the iterate MAP value and step size
    fprintf('Objective value : %g  \tit : %d\n',als_obj,t*iter_als)
    end
end
C_aux     =     C_aux*sqrt(norm_Y);
% Deleting zero error trailing at the end
als_vec(als_vec==0)   =     [];
als_vec   =     [0 iter_als:iter_als:iter_als*t;als_vec];
%-------------------------------------------------------------------------%    
end













