function [A_aux,B_aux,C_aux,A_p_aux,B_p_aux,als_vec] = ...
                complex_ccp_als(Y,Y_p,iter,sigma_n,A_0,B_0,C_0,A_p_0,B_p_0)
%-------------------------------------------------------------------------%
% [A_aux,B_aux,C_aux,A_p_aux,B_p_aux,als_vec] = ...
%               complex_ccp_als(Y,Y_p,iter,sigma_n,A_0,B_0,C_0,A_p_0,B_p_0)
%
% ALS algorithm for Coupled CP decomposition with complex factors. 
%
% The inputs are:
% 
% - Y         : first data matrix;
% - Y_p         : second data matrix;
% - iter        : maximum number of iterates;
% - sigma_n     : vector with noise standard deviations;
% - A_0,...     : initial factors for the first model;
% - A_p_0,...   : initial factors for the second model.  
%
% The outputs are:
% 
% - A,...       : factors for the first model;
% - A_p,...     : factors for the second model;
% - als_vec     : vector with the evolution of objective function.
%
% List of updates                 -     17/02/2016  -     R. C. Farias
%                                       Creation of the file 
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Number of iterations for each evaluation of the objective
iter_als  =     10;
% Size of the filter for the relative error decrease
n_diff_err=     3;
% Stopping criteria parameters
stop_diff =     eps;
stop_err  =     eps;
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Maximum number of iterations (/iter_als)
iter      =     ceil(iter/iter_als);
% Size of the tensor
dim_1     =     size(Y);
dim_2     =     size(Y_p);
% Index for n-mode matricization
J_1       =     prod(dim_1)./dim_1;
J_2       =     prod(dim_2)./dim_2;

% Three different unfoldings of the tensor
Y_mode    =     cell(3);
Y_p_mode  =     cell(3);
% Data tensors unfolding
% First tensor
Y_mode{1}       =     reshape(permute(Y,[1,2,3]),dim_1(1),J_1(1));  
Y_mode{2}       =     reshape(permute(Y,[2,1,3]),dim_1(2),J_1(2));
Y_mode{3}       =     reshape(permute(Y,[3,1,2]),dim_1(3),J_1(3));
% Second tensor
Y_p_mode{1}     =     reshape(permute(Y_p,[1,2,3]),dim_2(1),J_2(1));  
Y_p_mode{2}     =     reshape(permute(Y_p,[2,1,3]),dim_2(2),J_2(2));
Y_p_mode{3}     =     reshape(permute(Y_p,[3,1,2]),dim_2(3),J_2(3));

% Precisions and coupling strength
prec      =     1./(sigma_n.^2);

% Normalization of the MAP objective function function
norm      =     mean(prec);
% Normalized constants in the MAP objective function
prec      =     prec/norm;
% Tensors norms
norm_Y          =     sum(conj(Y(:)).*Y(:));
norm_Y_p        =     sum(conj(Y_p(:)).*Y_p(:));
%-------------------------------------------------------------------------%

%-----------------------Algorithm initialization--------------------------%
% Filter for the relative error decrease
diff_err_filt   =     1000*ones(1,n_diff_err);
% Storing initial condition on the compressed space
A_aux     =     A_0;
B_aux     =     B_0;
C_aux     =     C_0;
A_p_aux   =     A_p_0;
B_p_aux   =     B_p_0;
X_aux     =     A_aux*transpose(kr(C_aux,B_aux));
X_p_aux   =     A_p_aux*transpose(kr(C_aux,B_p_aux));

% Vector of squared norms of the error
als_vec   =           zeros(1,iter);
% Initial MAP objective value
als_old   =            prec(1)*sum(conj(Y(:)-X_aux(:)).*(Y(:)...
                       -X_aux(:)))+prec(2)*sum(conj(Y_p(:)-X_p_aux(:)).*...
                       (Y_p(:)-X_p_aux(:)));
% Vector of squared norms of the error
als_vec(1)=           als_old;
%-------------------------------------------------------------------------%

fprintf('\n Coupled matrix factorization ALS - CMF ALS \n\n')

%--------------------------Algorithm iterate------------------------------%
t         =     0;

while     (mean(diff_err_filt)>stop_diff &&  t<iter ...
          &&  als_old>stop_err   )

    % Iteration counter
    t=t+1;
    
    % Complex coupled ALS (10 iterates)
    for   i           =     1:iter_als
    
    % A factors update
    % A factor update
    temp_CB     =     (transpose(C_aux)*conj(C_aux)).*...
                      (transpose(B_aux)*conj(B_aux));
    A_aux       =     (Y_mode{1}*conj(kr(C_aux,B_aux)))/temp_CB;
    temp_CB_p   =     (transpose(C_aux)*conj(C_aux)).*...
                      (transpose(B_p_aux)*conj(B_p_aux));
    A_p_aux     =     (Y_p_mode{1}*conj(kr(C_aux,B_p_aux)))/temp_CB_p;
    % Normalization
    A_aux       =     A_aux.*repmat(1./A_aux(1,:),dim_1(1),1);
    A_p_aux     =     A_p_aux.*repmat(1./A_p_aux(1,:),dim_2(1),1);
    
    % B factors update
    temp_CA     =     (transpose(C_aux)*conj(C_aux)).*...
                      (transpose(A_aux)*conj(A_aux));
    B_aux       =     (Y_mode{2}*conj(kr(C_aux,A_aux)))/temp_CA;
    temp_CA_p   =     (transpose(C_aux)*conj(C_aux)).*...
                      (transpose(A_p_aux)*conj(A_p_aux));
    B_p_aux     =     (Y_p_mode{2}*conj(kr(C_aux,A_p_aux)))/temp_CA_p;
    % Normalization
    B_aux       =     B_aux.*repmat(1./B_aux(1,:),dim_1(2),1);
    B_p_aux     =     B_p_aux.*repmat(1./B_p_aux(1,:),dim_2(2),1);
    
    % Joint C factor update
    % C factor update
    temp_BA     =     (transpose(B_aux)*conj(B_aux)).*...
                      (transpose(A_aux)*conj(A_aux));
    Y_BA        =     Y_mode{3}*conj(kr(B_aux,A_aux));
    temp_BA_p   =     (transpose(B_p_aux)*conj(B_p_aux)).*...
                      (transpose(A_p_aux)*conj(A_p_aux));
    Y_BA_p      =     Y_p_mode{3}*conj(kr(B_p_aux,A_p_aux));
    C_aux       =     (Y_BA*prec(1)+Y_BA_p*prec(2))/...
                      (temp_BA*prec(1)+temp_BA_p*prec(2));
    end
    
    % Objective value
    err   =     real(norm_Y-2*real(sum(sum(Y_BA.*conj(C_aux))))...
                +sum(sum((transpose(C_aux)*conj(C_aux)).*temp_BA)));
    err_p =     real(norm_Y_p-2*real(sum(sum(Y_BA_p.*conj(C_aux))))...
                +sum(sum((transpose(C_aux)*conj(C_aux)).*temp_BA_p)));
    als_obj     =     prec(1)*err+prec(2)*err_p;
    
    % Evaluation of the relative error decrease
    diff_err_filt(1:end-1)  =     diff_err_filt(2:end);
    diff_err_filt(end)      =     abs((als_obj-als_old)/als_vec(1));
    
    % Storing the last error
    als_old        =     als_obj;
    als_vec(t+1)   =     als_obj;
    if mod(t*iter_als,100)  ==    0
    % Printing the iterate MAP value and step size
    fprintf('Objective value : %g  \tit : %d\n',als_obj,t*iter_als)
    end
end
% Normalization
C_aux     =     C_aux .* repmat(A_aux(1,:).*B_aux(1,:), dim_1(3),1);
A_aux     =     A_aux.*repmat(1./A_aux(1,:),dim_1(1),1);
A_p_aux   =     A_p_aux.*repmat(1./A_p_aux(1,:),dim_2(1),1);
B_aux     =     B_aux.*repmat(1./B_aux(1,:),dim_1(2),1);
B_p_aux   =     B_p_aux.*repmat(1./B_p_aux(1,:),dim_2(2),1);
% Deleting zero error trailing at the end
als_vec(als_vec==0)   =     [];
als_vec   =     [0 iter_als:iter_als:iter_als*t;als_vec];
%-------------------------------------------------------------------------%    
end













