function [CRB_C,CRB_AB,D_C,D_AB,D_C_AB]    =  complex_crb_cp(A,B,C,sigma_n)
%-------------------------------------------------------------------------%
% [CRB_C,CRB_AB,D_C,D_AB,D_AB_C]   =   complex_crb_cp(A,B,C,sigma_n)
%
% Cramr-Rao bound for the complex CP decomposition under additive complex 
% circular Gaussian noise.
%
% The inputs are:
% 
% - A,B,C       : complex factors of CP model;
% - sigma_n     : standard deviation of noise (same for real and imag.).
%
% The outputs are:
% 
% - CRB_C       : Cramr-Rao bound for vectorized coupled factor C.
%                 [vec(Re(C));vec(Im(C))]
% - CRB_AB      : Cramr-Rao bound for vectorized uncoupled factors
%                 A and B. [vec(Re(A));vec(Im(A));vec(Re(B)) vec(Im(B))]
% - D_C         : Fisher information block for vectorized coupled factor C.
% - D_AB        : Fisher information block for vectorized uncoupled factors
%                 A and B.
% - D_C_AB      : Cross Fisher information block between vectorized C and
%                 vectorized A and B.
%
% List of updates                 -     17/02/2012  -     R. C. Farias and
%                                                         C. Ren
%                                       Creation of the file 
%-------------------------------------------------------------------------%

%-------------------------------Parameters--------------------------------%
% Tensor dimensions
dim             =     [size(A,1) size(B,1) size(C,1)];
R               =     size(A,2);
% Matrices relating the vectorizations of the unfoldings
[~,J_13,J_23]   =     vec_unfold(dim);
%-------------------------------------------------------------------------%

%---------------------------------FI and CRB------------------------------%
% Matrices related to real and imaginary parts
% Coupled factor C
P_C =     [eye(dim(3)*R) 1i*eye(dim(3)*R)];
% Uncoupled factors A and B
P_AB=     [eye(dim(1)*R) 1i*eye(dim(1)*R) zeros(dim(1)*R,2*dim(2)*R);...
           zeros(dim(2)*R,2*dim(1)*R) eye(dim(2)*R) 1i*eye(dim(2)*R)];
% Matrices related to the Jacobian
% Factor C
Q   =     kron(kr(B,A),eye(dim(3)));
% Factor A
S_A =     J_13*(kron(kr(C,B),eye(dim(1))));
% Factor B
S_B =     J_23*(kron(kr(C,A),eye(dim(2))));
% Fisher information blocks
% Coupled factor C
D_C =     (2/(sigma_n^2))*real((P_C')*(Q')*Q*P_C);
% Uncoupled factors A and B
% Cross Fisher information block
D_C_AB    =     (2/(sigma_n^2))*real((P_C')*(Q')*[S_A S_B]*P_AB);
% Fisher information for uncoupled factors
D_AB      =     (2/(sigma_n^2))*real((P_AB')*([S_A S_B]')*[S_A S_B]*P_AB);
% Deleting rows and columns of known elements
I   =     [1:dim(1):2*dim(1)*R,2*dim(1)*R+1:dim(2):2*(dim(1)+dim(2))*R];
D_C_AB(:,I)     =     [];     
D_AB(I,:)       =     [];   D_AB(:,I)  =     [];
% Evaluating the Cramr-Rao bounds
% Coupled factor C
CRB_C     =     inv(D_C-(D_C_AB*(D_AB\transpose(D_C_AB))));
% Uncoupled factors A and B
CRB_AB    =     inv(D_AB-(transpose(D_C_AB)*(D_C\D_C_AB)));
%-------------------------------------------------------------------------%
end