%--------------------------similar_factors.m------------------------------%
%
% This file evaluates the performance of hard coupled complex CP (Complex
% CCP ALS) against uncoupled CP (ALS). The tensors are measured with 
% complex i.i.d. additive Gaussian noise.
%
% The results are also compared to Cramer-Rao bounds (CRB) for coupled and 
% uncoupled models.
%
% These simulations are related to Table 1 of the paper
% "Performance bounds for coupled models".
%
%-------------------------------------------------------------------------%

%--------------Cleaning the workspace and loading functions---------------%
clc;
clear     all;
close     all;
% Add path
addpath('COMPLEX_CRB_CCP','COMPLEX_CCP_ALS');
%-------------------------------------------------------------------------%

%-----------------------------Model parameters----------------------------%
% 3-way block dimensions
dim       =     [15 15 15];
dim_p     =     [15 15 dim(3)];
% Number of components
R         =     3;
% Signal to noise ratio (SNR) for the first tensor
SNR       =     0:5:40;
% SNR for the second tensor
SNR_p     =     20;
% Number of iterations for coupled ALS
iter_c    =     2000;
% Number of iterations for uncoupled ALS
iter_u    =     2000;
% Number of realizations for the evaluation of the sum of the MSE
n_real    =     500;
% Number of realizations for the initializations
n_init    =     5;
%-------------------------------------------------------------------------%

%------------------------------Tensor generation--------------------------%
% Fixing the seed to have the same tensor
rng(107)
% Random factors generation
A         =     (randn(dim(1),R)+1i*randn(dim(1),R))/sqrt(2);
% % A with two correlated columns
% A(:,2)    =     0.93*A(:,1)+...
%                 0.1*(randn(dim(1),1)+1i*randn(dim(1),1))/sqrt(2);
B         =     (randn(dim(2),R)+1i*randn(dim(2),R))/sqrt(2);
C         =     (randn(dim(3),R)+1i*randn(dim(3),R))/sqrt(2);
A_p       =     (randn(dim_p(1),R)+1i*randn(dim_p(1),R))/sqrt(2);
B_p       =     (randn(dim_p(2),R)+1i*randn(dim_p(2),R))/sqrt(2);

% Normalization of the first rows
A(1,:)    =     ones(1,R);
B(1,:)    =     ones(1,R);
A_p(1,:)  =     ones(1,R);
B_p(1,:)  =     ones(1,R);

% Tensors
X         =     reshape(A*transpose(kr(C,B)),dim);
X_p       =     reshape(A_p*transpose(kr(C,B_p)),dim_p);
%-------------------------------------------------------------------------%

%------------Simulation for different SNR of the first tensor-------------%
% Progress
N         =     n_real*n_init*length(SNR);
n         =     0;

% Complex noise standard deviation
sigma_n   =     [sqrt(R*10.^(-SNR/10))' ...
                sqrt(R*10^(-SNR_p/10))*ones(length(SNR))];

% Allocation for the squared errors
se_rA_u(dim(1)*R-R,n_real,length(SNR))        =     0;
se_iA_u(dim(1)*R-R,n_real,length(SNR))        =     0;
se_rA_c(dim(1)*R-R,n_real,length(SNR))        =     0;
se_iA_c(dim(1)*R-R,n_real,length(SNR))        =     0;
se_rB_u(dim(2)*R-R,n_real,length(SNR))        =     0;
se_iB_u(dim(2)*R-R,n_real,length(SNR))        =     0;
se_rB_c(dim(2)*R-R,n_real,length(SNR))        =     0;
se_iB_c(dim(2)*R-R,n_real,length(SNR))        =     0;
se_rA_p_u(dim_p(1)*R-R,n_real,length(SNR))    =     0;
se_iA_p_u(dim_p(1)*R-R,n_real,length(SNR))    =     0;
se_rA_p_c(dim_p(1)*R-R,n_real,length(SNR))    =     0;
se_iA_p_c(dim_p(1)*R-R,n_real,length(SNR))    =     0;
se_rB_p_u(dim_p(2)*R-R,n_real,length(SNR))    =     0;
se_iB_p_u(dim_p(2)*R-R,n_real,length(SNR))    =     0;
se_rB_p_c(dim_p(2)*R-R,n_real,length(SNR))    =     0;
se_iB_p_c(dim_p(2)*R-R,n_real,length(SNR))    =     0;
se_rC_u(dim(3)*R,n_real,length(SNR))          =     0;
se_iC_u(dim(3)*R,n_real,length(SNR))          =     0;
se_rC_c(dim(3)*R,n_real,length(SNR))          =     0;
se_iC_c(dim(3)*R,n_real,length(SNR))          =     0;
se_rC_p_u(dim(3)*R,n_real,length(SNR))        =     0;
se_iC_p_u(dim(3)*R,n_real,length(SNR))        =     0;
% Allocation for mean squared errors (MSE) and total MSE
mse_rA_u(length(SNR))             =     0;
mse_iA_u(length(SNR))             =     0;
mse_rB_u(length(SNR))             =     0;
mse_iB_u(length(SNR))             =     0;
mse_rC_u(length(SNR))             =     0;
mse_iC_u(length(SNR))             =     0;
mse_rA_p_u(length(SNR))           =     0;
mse_iA_p_u(length(SNR))           =     0;
mse_rB_p_u(length(SNR))           =     0;
mse_iB_p_u(length(SNR))           =     0;
mse_rC_p_u(length(SNR))           =     0;
mse_iC_p_u(length(SNR))           =     0;
mse_rA_c(length(SNR))             =     0;
mse_iA_c(length(SNR))             =     0;
mse_rB_c(length(SNR))             =     0;
mse_iB_c(length(SNR))             =     0;
mse_rC_c(length(SNR))             =     0;
mse_iC_c(length(SNR))             =     0;
mse_rA_p_c(length(SNR))           =     0;
mse_iA_p_c(length(SNR))           =     0;
mse_rB_p_c(length(SNR))           =     0;
mse_iB_p_c(length(SNR))           =     0;
total_mse_C_u(length(SNR))        =     0;
total_mse_C_p_u(length(SNR))      =     0;
total_mse_AB_u(length(SNR))       =     0;
total_mse_AB_p_u(length(SNR))     =     0;
total_mse_C_c(length(SNR))        =     0;
total_mse_C_p_c(length(SNR))      =     0;
total_mse_AB_c(length(SNR))       =     0;
total_mse_AB_p_c(length(SNR))     =     0;
% Allocation for the CRB on total MSE
total_CRB_C_u(length(SNR))        =     0;
total_CRB_C_p_u(length(SNR))      =     0;
total_CRB_AB_u(length(SNR))       =     0;
total_CRB_AB_p_u(length(SNR))     =     0;
total_CRB_C_c(length(SNR))        =     0;
total_CRB_AB_c(length(SNR))       =     0;
total_CRB_AB_p_c(length(SNR))     =     0;

% Change the seed to have different noise realizations
rng('shuffle')
for k     =     1:length(SNR)
    
%----------------Complex CRB for coupled and uncoupled models-------------%
% Uncoupled CRB for the first CP model
[CRB_C_u,CRB_AB_u,D_C,D_AB,D_C_AB]=     complex_crb_cp(A,B,C,sigma_n(k,1));
% Uncoupled CRB for the first CP model
[CRB_C_p_u,CRB_AB_p_u,D_C_p,D_AB_p,D_C_AB_p]=     ...
                                    complex_crb_cp(A_p,B_p,C,sigma_n(k,2));
% Coupled CRB
[CRB_C_c,CRB_C_p_c,CRB_AB_c,CRB_AB_p_c,K_C,K_C_p,K_AB,K_AB_p] =     ...
                                  complex_crb_ccp(CRB_C_u,CRB_AB_u,D_AB,...
                              D_C_AB,CRB_C_p_u,CRB_AB_p_u,D_AB_p,D_C_AB_p);
%-------------------------------------------------------------------------%
    
%------------------------Different noise realizations---------------------%
for j     =     1:n_real

%Noisy measurements
% First noisy tensor
Y         =     X+(sigma_n(k,1)/sqrt(2))*(randn(dim)+1i*randn(dim));
% Second noisy tensor
Y_p       =     X_p+(sigma_n(k,2)/sqrt(2))*(randn(dim_p)+1i*randn(dim_p));

% Different initializations
% Initial cost function values for picking best initialization
obj_n_val_1     =     10^50;
obj_n_val_2     =     10^50;
obj_c_n_val     =     10^50;
for i     =     1:n_init
%-----------------------------Initialization------------------------------%
A_0       =     (randn(size(A))+1i*randn(size(A)))/sqrt(2);
A_0(1,:)  =     A(1,:);
B_0       =     (randn(size(B))+1i*randn(size(B)))/sqrt(2);
B_0(1,:)  =     B(1,:);
   
A_p_0     =     (randn(size(A_p))+1i*randn(size(A_p)))/sqrt(2);
A_p_0(1,:)=     A_p(1,:);
B_p_0     =     (randn(size(A_p))+1i*randn(size(A_p)))/sqrt(2);
B_p_0(1,:)=     B_p(1,:);
 
C_0       =     (randn(size(C))+1i*randn(size(C)))/sqrt(2);
C_p_0     =     (randn(size(C))+1i*randn(size(C)))/sqrt(2);
%-------------------------------------------------------------------------%

%----------------------Uncoupled complex CP ALS---------------------------%
% Complex ALS for the first tensor
[A_n_u,B_n_u,C_n_u,obj_val_vec_1] =     complex_als(Y,iter_u,A_0,B_0,C_0);
% Complex ALS for the second tensor
[A_p_n_u,B_p_n_u,C_p_n_u,obj_val_vec_2] =     ...
                                 complex_als(Y_p,iter_u,A_p_0,B_p_0,C_p_0);
[A_p_n_u,B_p_n_u,C_p_n_u]   =    ...
                             amb_correct_1f(A_p_n_u,B_p_n_u,C_p_n_u,C_n_u);
if obj_val_vec_1(end)<obj_n_val_1
obj_n_val_1     =     obj_val_vec_1(end);
A_u             =     A_n_u;
B_u             =     B_n_u;
C_u             =     C_n_u;
end
if obj_val_vec_2(end)<obj_n_val_2
obj_n_val_2     =     obj_val_vec_2(end);
A_p_u           =     A_p_n_u;
B_p_u           =     B_p_n_u;
C_p_u           =     C_p_n_u;
end
%-------------------------------------------------------------------------%

%-----------------------Coupled complex CP ALS----------------------------%
% Initialization with warm start
% A_0       =     A_n_u;
% B_0       =     B_u;
% if        SNR(k)<=SNR_p
% C_0       =     C_p_n_u;
% else
% C_0       =     C_n_u;
% end
% A_p_0     =     A_p_n_u;
% B_p_0     =     B_p_n_u;
% C_p_0     =     C_p_n_u;

A_0       =     A;
B_0       =     B;
C_0       =     C;
A_p_0     =     A_p;
B_p_0     =     B_p;
C_p_0     =     C;
% Complex CCP_ALS
[A_n_c,B_n_c,C_n_c,A_p_n_c,B_p_n_c,obj_c_val_vec] = ...
                              complex_ccp_als(Y,Y_p,iter_c,sigma_n(k,:),...
                                                  A_0,B_0,C_0,A_p_0,B_p_0);
% Picking best result for different initializations
if obj_c_val_vec(end)<obj_c_n_val
obj_c_n_val     =     obj_c_val_vec(end);
A_c             =     A_n_c;
B_c             =     B_n_c;
C_c             =     C_n_c;
A_p_c           =     A_p_n_c;
B_p_c           =     B_p_n_c;
end
%-------------------------------------------------------------------------%
n=n+1;
clc
fprintf('Progress %g %%',(n/N)*100)
end

%--------Correcting the ambiguity and evaluating the squared error--------%
% Uncoupled CP ALS
% Correcting permutation and sign ambiguity for the first tensor
[A_u,B_u,C_u]         =     amb_correct(A_u,B_u,C_u,A,B,C);
% Correcting permutation and sign ambiguity for the second tensor
[A_p_u,B_p_u,C_p_u]   =...
                      amb_correct(A_p_u,B_p_u,C_p_u,A_p,B_p,C);                
% Squared errors on the factors
err =     real(A(2:end,:))-real(A_u(2:end,:));
se_rA_u(:,j,k)        =     (err(:)).^2;
err =     imag(A(2:end,:))-imag(A_u(2:end,:));
se_iA_u(:,j,k)        =     (err(:)).^2;
err =     real(B(2:end,:))-real(B_u(2:end,:));
se_rB_u(:,j,k)        =     (err(:)).^2;
err =     imag(B(2:end,:))-imag(B_u(2:end,:));
se_iB_u(:,j,k)        =     (err(:)).^2;
se_rC_u(:,j,k)        =     (real(C(:))-real(C_u(:))).^2;
se_iC_u(:,j,k)        =     (imag(C(:))-imag(C_u(:))).^2;
err =     real(A_p(2:end,:))-real(A_p_u(2:end,:));
se_rA_p_u(:,j,k)      =     (err(:)).^2;
err =     imag(A_p(2:end,:))-imag(A_p_u(2:end,:));
se_iA_p_u(:,j,k)      =     (err(:)).^2;
err =     real(B_p(2:end,:))-real(B_p_u(2:end,:));
se_rB_p_u(:,j,k)      =     (err(:)).^2;
err =     imag(B_p(2:end,:))-imag(B_p_u(2:end,:));
se_iB_p_u(:,j,k)      =     (err(:)).^2;
se_rC_p_u(:,j,k)      =     (real(C(:))-real(C_p_u(:))).^2;
se_iC_p_u(:,j,k)      =     (imag(C(:))-imag(C_p_u(:))).^2;
% Complex coupled CP ALS
% Correcting permutation and sign ambiguity for the first tensor
[A_c,B_c,C_c]   =     amb_correct(A_c,B_c,C_c,A,B,C);
% Correcting permutation and sign ambiguity for the second tensor
[A_p_c,B_p_c,~] =     amb_correct(A_p_c,B_p_c,C_c,A_p,B_p,C);
% Squared errors on the factors
err =     real(A(2:end,:))-real(A_c(2:end,:));
se_rA_c(:,j,k)        =     (err(:)).^2;
err =     imag(A(2:end,:))-imag(A_c(2:end,:));
se_iA_c(:,j,k)        =     (err(:)).^2;
err =     real(B(2:end,:))-real(B_c(2:end,:));
se_rB_c(:,j,k)        =     (err(:)).^2;
err =     imag(B(2:end,:))-imag(B_c(2:end,:));
se_iB_c(:,j,k)        =     (err(:)).^2;
se_rC_c(:,j,k)        =     (real(C(:))-real(C_c(:))).^2;
se_iC_c(:,j,k)        =     (imag(C(:))-imag(C_c(:))).^2;
err =     real(A_p(2:end,:))-real(A_p_c(2:end,:));
se_rA_p_c(:,j,k)      =     (err(:)).^2;
err =     imag(A_p(2:end,:))-imag(A_p_c(2:end,:));
se_iA_p_c(:,j,k)      =     (err(:)).^2;
err =     real(B_p(2:end,:))-real(B_p_c(2:end,:));
se_rB_p_c(:,j,k)      =     (err(:)).^2;
err =     imag(B_p(2:end,:))-imag(B_p_c(2:end,:));
se_iB_p_c(:,j,k)      =     (err(:)).^2;
%-------------------------------------------------------------------------%
end

%-------------------------------Results-----------------------------------%
% Mean squared errors on the factors
% Uncoupled complex CP ALS
mse_rA_u(k)     =     sum(mean(se_rA_u(:,:,k),2));
mse_iA_u(k)     =     sum(mean(se_iA_u(:,:,k),2));
mse_rB_u(k)     =     sum(mean(se_rB_u(:,:,k),2));
mse_iB_u(k)     =     sum(mean(se_iB_u(:,:,k),2));
mse_rC_u(k)     =     sum(mean(se_rC_u(:,:,k),2));
mse_iC_u(k)     =     sum(mean(se_iC_u(:,:,k),2));
mse_rA_p_u(k)   =     sum(mean(se_rA_p_u(:,:,k),2));
mse_iA_p_u(k)   =     sum(mean(se_iA_p_u(:,:,k),2));
mse_rB_p_u(k)   =     sum(mean(se_rB_p_u(:,:,k),2));
mse_iB_p_u(k)   =     sum(mean(se_iB_p_u(:,:,k),2));
mse_rC_p_u(k)   =     sum(mean(se_rC_p_u(:,:,k),2));
mse_iC_p_u(k)   =     sum(mean(se_iC_p_u(:,:,k),2));
% Coupled complex CP ALS
mse_rA_c(k)     =     sum(mean(se_rA_c(:,:,k),2));
mse_iA_c(k)     =     sum(mean(se_iA_c(:,:,k),2));
mse_rB_c(k)     =     sum(mean(se_rB_c(:,:,k),2));
mse_iB_c(k)     =     sum(mean(se_iB_c(:,:,k),2));
mse_rC_c(k)     =     sum(mean(se_rC_c(:,:,k),2));
mse_iC_c(k)     =     sum(mean(se_iC_c(:,:,k),2));
mse_rA_p_c(k)   =     sum(mean(se_rA_p_c(:,:,k),2));
mse_iA_p_c(k)   =     sum(mean(se_iA_p_c(:,:,k),2));
mse_rB_p_c(k)   =     sum(mean(se_rB_p_c(:,:,k),2));
mse_iB_p_c(k)   =     sum(mean(se_iB_p_c(:,:,k),2));

% Total MSE on coupled and uncoupled parameters for an uncoupled model
% Coupled factors
total_mse_C_u(k)      =     mse_rC_u(k)+mse_iC_u(k);
total_mse_C_p_u(k)    =     mse_rC_p_u(k)+mse_iC_p_u(k);
% Uncoupled factors
total_mse_AB_u(k)     =     mse_rA_u(k)+mse_iA_u(k)+...
                            mse_rB_u(k)+mse_iB_u(k);
total_mse_AB_p_u(k)   =     mse_rA_p_u(k)+mse_iA_p_u(k)+...
                            mse_rB_p_u(k)+mse_iB_p_u(k);
% Total MSE on coupled and uncoupled parameters for a coupled model
% Coupled factors
total_mse_C_c(k)      =     mse_rC_c(k)+mse_iC_c(k);
% Uncoupled factors
total_mse_AB_c(k)     =     mse_rA_c(k)+mse_iA_c(k)+...
                            mse_rB_c(k)+mse_iB_c(k);
total_mse_AB_p_c(k)   =     mse_rA_p_c(k)+mse_iA_p_c(k)+...
                            mse_rB_p_c(k)+mse_iB_p_c(k);
                        
% Total CRB on coupled and uncoupled parameters for an uncoupled model
% Coupled factors
total_CRB_C_u(k)      =     sum(diag(CRB_C_u));
total_CRB_C_p_u(k)    =     sum(diag(CRB_C_p_u));
% Uncoupled factors
total_CRB_AB_u(k)     =     sum(diag(CRB_AB_u));
total_CRB_AB_p_u(k)   =     sum(diag(CRB_AB_p_u));
% Total CRB on coupled and uncoupled parameters for a coupled model
% Coupled factors
total_CRB_C_c(k)      =     sum(diag(CRB_C_c));
% Uncoupled factors
total_CRB_AB_c(k)     =     sum(diag(CRB_AB_c));
total_CRB_AB_p_c(k)   =     sum(diag(CRB_AB_p_c));
end
%-------------------------------------------------------------------------%

%-----------------------------Results Fig. 1------------------------------%
% Coherency of the true factors
A_norm    =     A.*repmat(1./sqrt(real(sum(conj(A).*A))),dim(1),1);
coh_A     =     max(max(abs(A_norm'*A_norm)-eye(R)));
B_norm    =     B.*repmat(1./sqrt(real(sum(conj(B).*B))),dim(2),1);
coh_B     =     max(max(abs(B_norm'*B_norm)-eye(R)));
C_norm    =     C.*repmat(1./sqrt(real(sum(conj(C).*C))),dim(3),1);
coh_C     =     max(max(abs(C_norm'*C_norm)-eye(R)));
A_p_norm  =     A_p.*repmat(1./sqrt(real(sum(conj(A_p).*A_p))),dim_p(1),1);
coh_A_p   =     max(max(abs(A_p_norm'*A_p_norm)-eye(R)));
B_p_norm  =     B_p.*repmat(1./sqrt(real(sum(conj(B_p).*B_p))),dim_p(2),1);
coh_B_p   =     max(max(abs(B_p_norm'*B_p_norm)-eye(R)));
% Fig.1 (a) or (c) coupled factors
loglog(SNR,total_mse_C_u,'k',...
     SNR,total_mse_C_p_u,'k:',...
     SNR,total_mse_C_c,'k--',...
     SNR,total_CRB_C_u,'b',...
     SNR,total_CRB_C_p_u,'b:',...
     SNR,total_CRB_C_c,'b--')
legend('MSE C uncoupled','MSE C_p uncoupled','MSE C coupled',...
       'CRB C uncoupled','CRB C_p uncoupled','CRB C coupled')
xlabel('SNR (dB)')
ylabel('Total MSE')
% Fig.1 (b) or (d) uncoupled factors
figure
loglog(SNR,total_mse_AB_u,'k',...
     SNR,total_mse_AB_p_u,'k:',...
     SNR,total_mse_AB_c,'k--',...
     SNR,total_mse_AB_p_c,'k-.',...
     SNR,total_CRB_AB_u,'b',...
     SNR,total_CRB_AB_p_u,'b:',...
     SNR,total_CRB_AB_c,'b--',...
     SNR,total_CRB_AB_p_c,'b-.')
legend('MSE AB uncoupled','MSE AB_p uncoupled',...
       'MSE AB coupled','MSE AB_p coupled',...
       'CRB AB uncoupled','CRB AB_p uncoupled',...
       'CRB AB coupled','CRB AB_p coupled')
xlabel('SNR (dB)')
ylabel('Total MSE')
%-------------------------------------------------------------------------%